########################
# Instrumental Variables Models
# Dr. Justin Esarey
########################



# the Wald Estimator
####################

rm(list=ls())
set.seed(123456)
require(sem)

# Create an endogenous fake data set
W<-rbinom(10000, size=1, prob=0.5)  # this is the instrumental variable
Z<-runif(10000, min=-5, max=5)
y<-c()
X<-c()

# Y = 3 + X + 2Z + u1
# X = 4 + 2Y - 0.5Z + W + u2

# [ y  -x] = [ 3 + 2Z + u1]
# [ -2y x] = [ 4 - 0.5Z + W + u2]

# [1  -1] [y] = [3 + 2Z + u1]
# [-2  1] [x] = [4 - 0.5Z + W + u2]

for(i in 1:10000){
  mat<-matrix(data=c(1, -1, -2, 1), nrow=2, ncol=2, byrow=T)
  YY<-qr.solve(mat, b=c(3 + 2*Z[i] + rnorm(1, mean=0, sd=2), 4 - 0.5*Z[i] + W[i] + rnorm(1, mean=0, sd=2)))
  y[i]<-YY[1]
  X[i]<-YY[2]
  
}

# Try a standard OLS model
model<-lm(y~X+Z)
summary(model)

# the Wald estimator
dat<-data.frame(W,X,y,Z)
numerator <- mean(subset(dat, select=y, subset=W==1)$y) - mean(subset(dat, select=y, subset=W==0)$y)
denominator <- mean(subset(dat, select=X, subset=W==1)$X) - mean(subset(dat, select=X, subset=W==0)$X)
wald <- numerator / denominator
wald

# Try a 2SLS model
summary(tsls(y~X, ~W))
summary(tsls(y~X+Z, ~W+Z))





# IV/2SLS estimator
######################

rm(list=ls())
set.seed(123456)
require(sem)

# Create an endogenous fake data set
W<-runif(500, min=-5, max=5)	# this is the instrumental variable
y<-c()
X<-c()

# y = 3 + X + u1
# X = 4 + W + 2Y + u2

# [ y  -x] = [ 3 + u1]
# [ -2y x] = [ 4 + W + u2]

for(i in 1:500){
	mat<-matrix(data=c(1, -1, -2, 1), nrow=2, ncol=2, byrow=T)
	YY<-qr.solve(mat, b=c(3 + rnorm(1, mean=0, sd=2), 4 + W[i]+rnorm(1, mean=0, sd=2)))
	y[i]<-YY[1]
	X[i]<-YY[2]

}

# Try a standard OLS model
model<-lm(y~X)
summary(model)

# Try a 2SLS model
summary(tsls(y~X, ~W))

library(foreign)
dat<-data.frame(y=y, X=X, W=W)
write.dta(dat, file="sample-iv-data.dta")


# more exogenous variables
############################

rm(list=ls())
set.seed(123456)
require(sem)

# Create an endogenous fake data set
W<-runif(2000, min=-5, max=5)  # this is the instrumental variable
Z<-runif(2000, min=-5, max=5)
y<-c()
X<-c()

# y = 3 + X + 2Z + u1
# X = 4 + 2Y + W - 0.5Z + u2

# [ y  -x] = [ 3 + 2Z + u1]
# [ -2y x] = [ 4 - 0.5Z + W + u2]

for(i in 1:2000){
  mat<-matrix(data=c(1, -1, -2, 1), nrow=2, ncol=2, byrow=T)
  YY<-qr.solve(mat, b=c(3 + 2*Z[i] + rnorm(1, mean=0, sd=2), 4 - 0.5*Z[i] + W[i] + rnorm(1, mean=0, sd=2)))
  y[i]<-YY[1]
  X[i]<-YY[2]
  
}

library(foreign)
dat<-data.frame(y=y, X=X, W=W, Z=Z)
write.dta(dat, file="sample-iv-data-3.dta")

# Try a standard OLS model
model<-lm(y~X+Z)
summary(model)

# Try a 2SLS model
summary(tsls(y~X+Z, ~W)) # does not work!
summary(tsls(y~X, ~W))
summary(tsls(y~X+Z, ~Z+W))







# more instrumental variables
#############################

rm(list=ls())
set.seed(4532)
require(sem)

# Create an endogenous fake data set
K<-runif(500, min=-5, max=5)	# this is one instrumental variable for y
L<-runif(500, min=-5, max=5)	# this is one instrumental variable for y
M<-runif(500, min=-5, max=5)	# this is one instrumental variable for X
N<-runif(500, min=-5, max=5)	# this is one instrumental variable for X
Z<-runif(500, min=-5, max=5)	# a common exogenous variable
y<-c()
X<-c()


# y = 3 + X + 2Z - K + 0.25L + u1
# X = 4 - 2*Y + 0.5Z + 3M - 0.1N + u2

# [ y  -x] = [ 3 + 2Z  + K + 0.25L + u1]
# [ 2y x] = [ 4 - 0.5Z + 3M - 0.1N + u2]



for(i in 1:500){
	mat<-matrix(data=c(1, -1, 2, 1), nrow=2, ncol=2, byrow=T)
	YY<-qr.solve(mat, b=c(3 + 2*Z[i] + K[i] + 0.25*L[i] + rnorm(1, mean=0, sd=1), 4 - 0.5*Z[i] + 3*M[i] - 0.1*N[i] + rnorm(1, mean=0, sd=1)))
	y[i]<-YY[1]
	X[i]<-YY[2]

}

# Try a standard OLS model
model<-lm(y~X+K+L+Z)
summary(model)

# Try a 2SLS model
summary(tsls(y~X, ~M))
summary(tsls(y~X, ~M+N))
summary(tsls(y~X+K+L+Z, ~M+K+L+Z))
summary(tsls(y~X+K+L+Z, ~M+N+K+L+Z))

library(foreign)
dat<-data.frame(y=y, X=X, K=K, L=L, Z=Z, M=M, N=N, K=K, L=L, Z=Z)
write.dta(dat, file="sample-iv-data-2.dta")


# Sargan test for overidentification
resid <- residuals(tsls(y~X+K+L+Z, ~M+N+K+L+Z))
eTz <- sum(predict(lm(resid~M+N+K+L+Z))^2)
sargan.s <- eTz/(sum(resid^2)/length(resid))
1-pchisq(sargan.s, df=1)






# Measurement Error
######################

rm(list=ls())
set.seed(123456)
require(sem)

X<-runif(200, min=-10, max=10)
X1<-X+rnorm(200, mean=0, sd=3)

pr.W<-pnorm(X, mean=0, sd=2)
W<-as.numeric(runif(200)<pr.W)

y<-2 + 3*X + rnorm(200, mean=0, sd=2)

# use measurement only
summary(lm(y~X1))
# Try a 2SLS model
summary(tsls(y~X1, ~W))




set.seed(123456)
X<-runif(2000, min=-10, max=10)
X1<-X+rnorm(2000, mean=0, sd=3)

pr.W<-pnorm(X, mean=0, sd=2)
W<-as.numeric(runif(2000)<pr.W)

y<-2 + 3*X + rnorm(2000, mean=0, sd=2)

# use measurement only
summary(lm(y~X1))
# Try a 2SLS model
summary(tsls(y~X1, ~W))








# Omitted Variable Bias
#########################

rm(list=ls())
set.seed(123456)

Z<-as.numeric(runif(500)<0.5)
W<-as.numeric(runif(500)<0.5)
X<-rnorm(500, mean=2*Z+W, sd=1)
y<-2 + 3*X + 2*Z + rnorm(500, mean=0, sd=1)

summary(lm(y~X))
summary(lm(y~X+Z))

# Try a 2SLS model
summary(tsls(y~X, ~W))
summary(tsls(y~X+Z, ~W+Z))


